#! /usr/bin/env python
# -*- coding: utf-8 -*-

'''
This package is to provide use usable interfaces
'''

from pydatatest.common.testcase import PyDataTestCase

from pydatatest.common.inject import inject_def
from pydatatest.common.inject import inject
from pydatatest.common.inject import InjectError


from pydatatest.common.runner import runner
from pydatatest.common.runner import run_with
from pydatatest.common.runner import test
from pydatatest.common.runner import test_runner



# testcase = PyDataTestCase # a signle test case


# def run_with():
#     '''
#     indicates the runner which will run current testcase\
#     '''
#     pass

# def test():
#     '''
#     register a testcase
#     '''
#     pass


#! /usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from common.inject import _inject_multi_to


class TestError(Exception):
    pass


class PyDataTestCase(unittest.TestCase):
    '''test base case
    '''
    def __init__(self, methodName):
        super().__init__(methodName)
        self._pydatatest_runner = None
        self._pydatatest_multi = False
        self._pydatatest_dataset = []

    @classmethod
    def before_all(cls):
        '''
        before all tests
        '''
        pass

    @classmethod
    def after_all(cls):
        '''
        after all tests
        '''
        pass
    
    def before_each(self):
        '''
        run before each test method
        '''
        pass

    def after_each(self):
        '''
        run after each test method
        '''
        pass
    
    def before_each_data(_self):
        '''
        run before each test data
        '''
        pass

    def after_each_data(self):
        '''
        run after each test data
        '''
        pass

    def run(self, result=None):
        '''
        run a testcase method
        '''
        e = None
        if result is not None:
            if hasattr(result, 'dots'):
                result.dots = False

        self.before_each_data()
        # try:
        super().run(result)
        # except Exception as e:
        #     self.after_each_data()
        #     pass
        self.after_each_data()

        if (self._pydatatest_multi):
            for i in range(len(self._pydatatest_dataset)):
                print("dataset in self", self._pydatatest_dataset)
                # _inject_multi_to(self._pydatatest_dataset, self)
                self.before_each_data()
                # try:
                super().run(result)
                print(result)
                # except Exception as e:
                #     self.after_each_data()
                #     break
                self.after_each_data()
            # multi = self._pydatatest_multi
            # dataset = self._pydatatest_dataset
            # while multi:
            #     print("===before inject::", dataset, multi)
            #     self.before_each_data()
            #     super().run(result)
            #     dataset = self._pydatatest_dataset
            #     multi = self._pydatatest_multi
            #     print("===after inject::", self._pydatatest_dataset, self._pydatatest_multi)
            #     self.after_each_data()
        if e is not None:
            raise TestError(e)
    
    def setUp(self):
        self.before_each()

    def tearDown(self):
        self.after_each()

    @classmethod
    def setUpClass(cls):
        cls.before_all()

    @classmethod
    def tearDownClass(cls):
        cls.after_all()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import csv


def get_data(file_name):
    data = []
    with open(file_name, 'r') as csv_file:
        data = list(csv.reader(csv_file, delimiter=' '))
    return data


#! /usr/bin/env python
# -*- coding: utf-8 -*-

import requests
import json
import functools


class Http():
    def post(self, url, data, headers=[]):
        result = requests.post(url=url, data=data).json()
        res = json.dumps(result, ensure_ascii=False, sort_keys=True, indent=2)
        return res

    def get(self, url, params, headers=[]):
        result = requests.get(url=url, params=params).json()
        res = json.dumps(result, ensure_ascii=False, sort_keys=True, indent=2)
        return res


def need_login(login_func):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kw):
            login_func()
            return func(*args, **kw)

        return wrapper

    return decorator



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from pydatatest.common import testcase
from pydatatest.util.inspect import retrieve_name


class Runner(object):
    '''
    Runner instance
    '''
    def __init__(self, name):
        self._pydatatest_tests = []
        self._pydatatest_name = name

    def run(self):
        test_suite = unittest.TestSuite()
        for test in self._pydatatest_tests:
            test_suite.addTest(unittest.makeSuite(test))
        unittest.TextTestRunner().run(test_suite)

    def add_test(self, test):
        if not isinstance(test, testcase.PyDataTestCase):
            raise Exception("type mismatch: expected to be a PyDataTest")
        self._pydatatest_tests.append(test)


runners = {}


def runner(name = ""):
    '''
    get a runner
    '''
    if not name:
        name = retrieve_name(name)
        if not name:
            import random
            import string
            name = 'runner_' + ''.join(random.sample(string.ascii_letters + string.digits, 10)) #生成5位随机字符，包括大小写字母和数字
    r = Runner(name)        
    runners.__setitem__(name, r)
    return r


def run_with(runner):
    '''
    declare a testcase that will run with a Runner
    '''
    if type(runner) == str:
        runner = runners.__getitem__(runner)
    def decorator(klass):
        klass._pydatatest_runner = runner
        return klass
    return decorator


def test(klass):
    '''
    register a testcase class or method
    '''
    runner = klass._pydatatest_runner
    runner._pydatatest_tests.append(klass)
    return klass


def test_runner(runner):
    '''
    pass test data to customised runner
    '''
    def decorator(func):
        def wrapper(*args, **kw):
            kw['_pydatatest_tests'] = func.self._pydatatest_runner.tests
            func(*args, **kw)
        return wrapper
    return decorator


#! /usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name= 'pydatatest',
    version= '1.0.0',
    package_dir={"": "src"},
    packages=find_packages('src'),
    # py_modules =['api', 'data', 'common'],
    author= 'aak1247',
    author_email='aak1247@126.com',
    url='https://github.com/aak1247/PyDataTest',

    description= 'Data driven test framework',

    install_requires=[
        "pyyaml"
    ],
)


import inspect


def retrieve_name(var):
    callers_local_vars = inspect.currentframe().f_back.f_locals.items()
    for var_name, var_value in callers_local_vars:
        if var_value is var:
            return var_name
    return ""


#! /usr/bin/env python
# -*- coding: utf-8 -*-

from inspect import isclass, isfunction, ismethod

class InjectError(Exception):
    pass

def _inject_to(data, self):
    '''
    actual inject method
    '''
    if not self or not hasattr(self, '_pydatatest_variables'):
        raise InjectError('inject variable not defined')
    for i, name in enumerate(self._pydatatest_variables):
        setattr(self, name, data[i])


def _inject_multi_to(dataset, self):
    '''
    inject multi data into one testcase method
    '''
    if not self or not hasattr(self, '_pydatatest_variables'):
        raise InjectError('inject variable not defined')
    if len(dataset) > 1:
        data = dataset[0]
        self._pydatatest_dataset = []
        self._pydatatest_dataset = dataset[1:]
        self._pydatatest_multi = True
        print("dataset:", self._pydatatest_dataset)
        print("should be:", dataset[1:])
    else:
        data = dataset[0]
        self._pydatatest_dataset = None
        self._pydatatest_multi = False
    print("data:", data)
    for i, name in enumerate(self._pydatatest_variables):
        setattr(self, name, data[i])


def _remove(self):
    '''
    delete injected data from runner after completed a testcase method
    '''
    _pydatatest_variables = self._pydatatest_variables
    for name in self._pydatatest_variables:
        delattr(self, name)

__injected = {}

def inject(data, multi=False):
    '''
    inject test data to a scope (a testcase method)
    '''
    def decorator(scope):
        if isfunction(scope):
            func = scope
            def wrapper(self):
                print("check multi start")
                if multi:
                    if func in __injected:
                        _inject_multi_to(self._pydatatest_dataset, self)
                    else:
                        _inject_multi_to(data, self)
                        __injected.__setitem__(func, 1)
                else:
                    _inject_to(data, self)
                func(self)
                print("check multi end")
                _remove(self)
            return wrapper
        else:
            raise InjectError("inject is allowed on testcase method")
    return decorator


def inject_def(title=[], session=False):
    '''
    declare inject data variable (used in a testcase class)
    '''
    def decorator(scope):
        if isclass(scope):
            klass = scope
            if session:
                import requests
                def setUpSession(func, klass):
                    def decorator(*args, **kw):
                        klass.session = requests.Session()
                        func(*args, **kw)
                        return klass
                    return decorator

                def tearDownSession(func, klass):
                    def decorator(*args, **kw):
                        klass.session.close()
                        func(*args, **kw)
                        return klass
                    return decorator
                
                klass.setUp = setUpSession(klass.setUp, klass)
                klass.tearDown = tearDownSession(klass.tearDown, klass)
            return type(klass.__name__, (klass, ), {"_pydatatest_variables": title})
        else:
            raise InjectError("inject_def is allowed on testcase class")
    return decorator